#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "non_gui_defs.h"

gdouble rnd(gdouble a,gdouble b)//generate a random number between a and b
{								
    gdouble randnr;

    srandom( random() );	//re-initialize the rnd nr generator
    randnr=(gfloat)random()/(gfloat)0x7fffffff;
    randnr=randnr*(b-a)+a;
	
    return(randnr);
}

/*generate a gauss-distributed random nr
  ("by Box and Muller, and recommended by Knuth")*/
gdouble gaussrand(void)
{
    static gdouble V1, V2, S;
    static gint phase = 0;
    gdouble X;

    if(phase == 0) {
	do {
	    gdouble U1 = (gdouble)rand() / RAND_MAX;
	    gdouble U2 = (gdouble)rand() / RAND_MAX;

	    V1 = 2 * U1 - 1;
	    V2 = 2 * U2 - 1;
	    S = V1 * V1 + V2 * V2;
	} while(S >= 1 || S == 0);

	X = V1 * sqrt(-2 * log(S) / S);
    } else
	X = V2 * sqrt(-2 * log(S) / S);

    phase = 1 - phase;

    return X;
}

/* Generate a gauss-distributed random nr using 'gaussrand';
it'll be with a probability of 99.7% between b and c
and at any case between a and d.
If b<a (d<c), there are no such restrictions */
gdouble gaussvt(gdouble a,gdouble b,gdouble c,gdouble d)
{						   
    gdouble result;				   
     	
    result = (gfloat)(c-b) / 6 * gaussrand()+(gfloat)(c+b) / 2;
    if(result < a && a <= b) result = a;
    if(result > d && d >= c) result = d;
    return result;
}

/* write a permutation of the integers between 'min' and 'max'
   into the array 'perm' */
gint write_permutation(gint *perm, gint min, gint max)
{
    gdouble x[max - min + 1];
    gint i,j,ok = 0;

    for(i=0;i<max-min+1;i++)
    {
	perm[i] = min - 1;
	x[i] = rnd(0,1);
    }

    for(i=0;i<max-min+1;i++)
    {
	for(j=i+1;j<max-min+1;j++)
	{
	    if(perm[i]==perm[j])
	    {
		ok = 1;
	    }
	}
    }

    while(ok != 0)
    {

	for(i=0;i<max-min+1;i++)
	{
	    for(j=i;j<max-min+1;j++)
	    {
		if(x[i]>=x[j]) perm[i]++;
		else perm[j]++;
	    }
	}

	ok = 0;

	for(i=0;i<max-min+1;i++)
	{
	    for(j=i+1;j<max-min+1;j++)
	    {
		if(perm[i]==perm[j])
		{
		    ok = 1;
		}
	    }
	}
    }

    return 0;
}

/* sum up the elements of an integer array between indices
   'min' and 'max' */
gfloat sum(gint *array, gint min, gint max)
{
    gint i;
    gfloat result = 0;
    
    for(i=min;i<=max;i++)
	result += array[i];

    return result; 
}

/* compute the smallast element of an array from element
 first to element last */
gint min_int(gint *array, gint first, gint last)
{
    gint i, result=array[first];

    for(i=first+1;i<=last;i++)
	if(array[i] < result)
	    result = array[i];

    return result;
}

gfloat min_float(gfloat *array, gint first, gint last)
{
    gint i;
    gfloat result=array[first];

    for(i=first+1;i<=last;i++)
	if(array[i] < result)
	    result = array[i];

    return result;
}
